@@ -3,14 +3,15 @@ import os
 import shutil
 import sys
 import warnings
-from collections import OrderedDict  # can be removed from python 3.7
 from distutils.version import LooseVersion
 from pathlib import Path
-from typing import Any, Dict, List, MutableMapping, Optional, Set, Union, cast
+from typing import Any, Dict, List, Optional, Set, cast
 
 import requests
 import typer
+from git import Repo as GitRepo
 from glom import glom
+from python_on_whales import docker
 
 from controller import (
     COMPOSE_ENVIRONMENT_FILE,
@@ -22,19 +23,23 @@ from controller import (
     PROJECT_DIR,
     PROJECTRC,
     SUBMODULES_DIR,
+    SWARM_MODE,
+    ComposeConfig,
+    EnvType,
     __version__,
-    gitter,
     log,
+    print_and_exit,
 )
 from controller.commands import load_commands
-from controller.compose import Compose
+from controller.deploy.compose import Compose
 from controller.packages import Packages
 from controller.project import ANGULAR, NO_FRONTEND, Project
 from controller.templating import Templating
-from controller.utilities import configuration, services, system
+from controller.utilities import configuration, git, services, system
 
 warnings.simplefilter("always", DeprecationWarning)
 
+# From python 3.8 it could be a TypedDict
 DataFileStub = Dict[str, List[str]]
 
 ROOT_UID = 0
@@ -43,21 +48,22 @@ BASE_UID = 1000
 
 class Configuration:
     projectrc: Dict[str, str] = {}
-    # To be better charactirized. This is a:
+    # To be better characterized. This is a:
     # {'variables': 'env': Dict[str, str]}
     host_configuration: Dict[str, Dict[str, Dict[str, str]]] = {}
-    specs: MutableMapping[str, str] = OrderedDict()
+    specs: Dict[str, str] = {}
     services_list: Optional[str]
+    excluded_services_list: Optional[str]
     environment: Dict[str, str]
 
     action: Optional[str] = None
 
     production: bool = False
     testing: bool = False
-    privileged: bool = False
     project: str = ""
     frontend: Optional[str] = None
     hostname: str = ""
+    remote_engine: Optional[str] = None
     stack: str = ""
     load_backend: bool = False
     load_frontend: bool = False
@@ -79,8 +85,12 @@ class Configuration:
     # It will be replaced with PROJECT_DIR/project
     ABS_PROJECT_PATH: Path = PROJECT_DIR
 
+    # used by single-container commands (interfaces, ssl, volatile, ...) in swarm mode
+    FORCE_COMPOSE_ENGINE: bool = False
+
     @staticmethod
     def set_action(action: Optional[str]) -> None:
+        log.debug("Requested command: {}", action)
         Configuration.action = action
         Configuration.initialize = Configuration.action == "init"
         Configuration.update = Configuration.action == "update"
@@ -126,7 +136,14 @@ def controller_cli_options(
         None,
         "--services",
         "-s",
-        help="Comma separated list of services",
+        help="Comma separated list of services to be included",
+        callback=projectrc_values,
+    ),
+    excluded_services_list: Optional[str] = typer.Option(
+        None,
+        "--skip-services",
+        "-S",
+        help="Comma separated list of services to be exluded",
         callback=projectrc_values,
     ),
     hostname: str = typer.Option(
@@ -166,12 +183,10 @@ def controller_cli_options(
         "-e",
         help="Temporary change the value of an environment variable",
     ),
-    privileged: bool = typer.Option(
-        False,
-        "--privileged",
-        help="Allow containers privileged mode",
-        callback=projectrc_values,
-        show_default=False,
+    remote_engine: Optional[str] = typer.Option(
+        None,
+        "--remote",
+        help="Execute commands on a remote host",
     ),
     no_backend: bool = typer.Option(
         False,
@@ -207,12 +222,18 @@ def controller_cli_options(
 
     Configuration.set_action(ctx.invoked_subcommand)
 
+    if services_list and excluded_services_list:
+        print_and_exit(
+            "Incompatibile use of both --services/-s and --skip-services/-S options"
+        )
+
     Configuration.services_list = services_list
+    Configuration.excluded_services_list = excluded_services_list
     Configuration.production = production
     Configuration.testing = testing
-    Configuration.privileged = privileged
     Configuration.project = project
     Configuration.hostname = hostname
+    Configuration.remote_engine = remote_engine
     Configuration.environment = {}
     for e in environment:
         key, value = e.split("=")
@@ -235,20 +256,16 @@ class CommandsData:
         files: List[Path] = [],
         base_files: List[Path] = [],
         services: List[str] = [],
-        services_list: Any = None,
         active_services: List[str] = [],
-        base_services: List[Any] = [],
-        compose_config: List[Any] = [],
-        services_dict: Dict[str, Any] = None,
+        base_services: ComposeConfig = {},
+        compose_config: ComposeConfig = {},
     ):
         self.files = files
         self.base_files = base_files
         self.services = services
-        self.services_list = services_list
         self.active_services = active_services or []
         self.base_services = base_services
         self.compose_config = compose_config
-        self.services_dict = services_dict or {}
 
 
 class Application:
@@ -263,7 +280,9 @@ class Application:
     controller: Optional["Application"] = None
     project_scaffold = Project()
     data = CommandsData()
-    gits: MutableMapping[str, gitter.GitRepoType] = OrderedDict()
+    # type ignore to be removed once the new gitpython version will be released
+    gits: Dict[str, GitRepo] = {}  # type: ignore
+    env: Dict[str, EnvType] = {}
 
     def __init__(self) -> None:
 
@@ -274,19 +293,13 @@ class Application:
         self.base_files: List[Path] = []
         self.services = None
         self.enabled_services: List[str] = []
-        self.base_services: List[Any] = []
-        self.compose_config: List[Any] = []
-        self.services_dict: Dict[str, List[Any]] = {}
+        self.base_services: ComposeConfig = {}
+        self.compose_config: ComposeConfig = {}
 
         load_commands()
 
         Application.load_projectrc()
 
-    @staticmethod
-    def exit(message: str, *args: Union[str, Path], **kwargs: Union[str, Path]) -> None:
-        log.critical(message, *args, **kwargs)
-        sys.exit(1)
-
     @staticmethod
     def get_controller() -> "Application":
         if not Application.controller:  # pragma: no cover
@@ -301,7 +314,7 @@ class Application:
         main_folder_error = Application.project_scaffold.check_main_folder()
 
         if main_folder_error:
-            Application.exit(main_folder_error)
+            print_and_exit(main_folder_error)
 
         if not Configuration.print_version:
             Application.check_installed_software()
@@ -371,8 +384,7 @@ class Application:
         self.make_env()
 
         # Compose services and variables
-        self.read_composers()
-        self.set_active_services()
+        self.get_compose_configuration()
 
         self.check_placeholders()
 
@@ -382,11 +394,9 @@ class Application:
             files=self.files,
             base_files=self.base_files,
             services=self.enabled_services,
-            services_list=Configuration.services_list,
             active_services=self.active_services,
             base_services=self.base_services,
             compose_config=self.compose_config,
-            services_dict=self.services_dict,
         )
 
         return None
@@ -394,9 +404,7 @@ class Application:
     @staticmethod
     def load_projectrc() -> None:
 
-        projectrc_yaml = configuration.load_yaml_file(
-            PROJECTRC, path=Path(), is_optional=True
-        )
+        projectrc_yaml = configuration.load_yaml_file(file=PROJECTRC, is_optional=True)
 
         Configuration.host_configuration = projectrc_yaml.pop(
             "project_configuration", {}
@@ -416,21 +424,47 @@ class Application:
 
         # 17.05 added support for multi-stage builds
         # https://docs.docker.com/compose/compose-file/compose-file-v3/#compose-and-docker-compatibility-matrix
+        # 18.09.2 fixed the CVE-2019-5736 vulnerability
+        # 20.10.0 introduced copy --chmod and improved logging
         Packages.check_program(
-            "docker", min_version="17.05", min_recommended_version="19.03.8"
+            "docker", min_version="20.10.0", min_recommended_version="20.10.0"
         )
+
+        if docker.buildx.is_installed():
+            v = docker.buildx.version()
+            log.debug("docker buildx is installed: {}", v)
+        else:  # pragma: no cover
+            print_and_exit(
+                "A mandatory dependency is missing: docker buildx not found"
+                "\nInstallation guide: https://github.com/docker/buildx#binary-release"
+                "\nor try the automated installation with rapydo install buildx"
+            )
+
+        if docker.compose.is_installed():
+            # NotImplementedError
+            # v = docker.compose.version()
+            log.debug("docker compose is installed")
+        else:  # pragma: no cover
+
+            if SWARM_MODE:
+                print_and_exit(
+                    "A mandatory dependency is missing: docker compose not found"
+                    "\nInstallation guide: "
+                    "https://docs.docker.com/compose/cli-command/#installing-compose-v2"
+                    "\nor try the automated installation with rapydo install compose"
+                )
+            else:
+                log.warning(
+                    "Docker Compose V2 will be soon mandatory and it is not installed"
+                    "\nInstallation guide: "
+                    "https://docs.docker.com/compose/cli-command/#installing-compose-v2"
+                    "\nor try the automated installation with rapydo install compose"
+                )
+
         Packages.check_program("git")
 
-        # Check for CVE-2019-5736 vulnerability
-        Packages.check_docker_vulnerability()
-
-        Packages.check_python_package("compose", min_version="1.18")
-        Packages.check_python_package("docker", min_version="4.0.0")
-        Packages.check_python_package("requests", min_version="2.6.1")
-        Packages.check_python_package("pip", min_version="10.0.0")
-
     def read_specs(self, read_extended: bool = True) -> None:
-        """ Read project configuration """
+        """Read project configuration"""
 
         try:
 
@@ -449,7 +483,7 @@ class Application:
             self.extended_project_path = confs[2]
 
         except AttributeError as e:  # pragma: no cover
-            Application.exit(str(e))
+            print_and_exit(str(e))
 
         Configuration.frontend = glom(
             Configuration.specs, "variables.env.FRONTEND_FRAMEWORK", default=NO_FRONTEND
@@ -475,7 +509,7 @@ class Application:
         )
 
         if not Configuration.rapydo_version:  # pragma: no cover
-            Application.exit(
+            print_and_exit(
                 "RAPyDo version not found in your project_configuration file"
             )
 
@@ -485,8 +519,9 @@ class Application:
     def preliminary_version_check() -> None:
 
         specs = configuration.load_yaml_file(
-            file=configuration.PROJECT_CONF_FILENAME,
-            path=Configuration.ABS_PROJECT_PATH,
+            file=Configuration.ABS_PROJECT_PATH.joinpath(
+                configuration.PROJECT_CONF_FILENAME
+            ),
             keep_order=True,
         )
 
@@ -512,43 +547,49 @@ class Application:
             return True
         else:  # pragma: no cover
             if r > c:
-                action = f"Upgrade your controller to version {r}"
+                ac = f"Upgrade your controller to version {r}"
             else:
-                action = f"Downgrade your controller to version {r}"
-                action += " or upgrade your project"
+                ac = f"Downgrade your controller to version {r} or upgrade your project"
 
-            msg = "RAPyDo version is not compatible\n\n"
-            msg += "This project requires rapydo {}, you are using {}\n\n{}\n".format(
-                r, c, action
-            )
+            msg = f"""RAPyDo version is not compatible.
 
-            log.critical(msg)
-            sys.exit(1)
+This project requires rapydo {r}, you are using {c}. {ac}
+
+You can use of one:
+  -  rapydo install               (install in editable from submodules/do, if available)
+  -  rapydo install --no-editable (install from pypi)
+
+"""
+
+            print_and_exit(msg)
 
     @staticmethod
     def check_internet_connection() -> None:
-        """ Check if connected to internet """
+        """Check if connected to internet"""
 
         try:
             requests.get("https://www.google.com")
             if Configuration.check:
                 log.info("Internet connection is available")
         except requests.ConnectionError:  # pragma: no cover
-            Application.exit("Internet connection is unavailable")
+            print_and_exit("Internet connection is unavailable")
 
-    # from_path: Optional[Path]
+    # type ignore to be removed once the new gitpython version will be released
     @staticmethod
-    def working_clone(name, repo, from_path=None):
+    def working_clone(  # type: ignore
+        name: str, repo: Dict[str, str], from_path: Optional[Path] = None
+    ) -> Optional[GitRepo]:
 
         # substitute values starting with '$$'
         myvars = {
             ANGULAR: Configuration.frontend == ANGULAR,
         }
-        repo = services.apply_variables(repo, myvars)
 
-        # Is this single repo enabled?
-        if not repo.pop("if", True):
-            return None
+        condition = repo.get("if", "")
+        if condition.startswith("$$"):
+            # Is this repo enabled?
+            if not myvars.get(condition.lstrip("$"), None):
+                return None
 
         repo.setdefault(
             "branch",
@@ -561,7 +602,7 @@ class Application:
 
             local_path = from_path.joinpath(name)
             if not local_path.exists():
-                Application.exit("Submodule {} not found in {}", name, local_path)
+                print_and_exit("Submodule {} not found in {}", name, local_path)
 
             submodule_path = Path(SUBMODULES_DIR, name)
 
@@ -574,85 +615,116 @@ class Application:
 
             os.symlink(local_path, submodule_path)
 
-        return gitter.clone(
-            url=repo.get("online_url"),
-            path=name,
-            branch=repo.get("branch"),
+        url = cast(str, repo.get("online_url"))
+        branch = cast(str, repo.get("branch"))
+        return git.clone(
+            url=url,
+            path=Path(name),
+            branch=branch,
             do=Configuration.initialize,
             check=not Configuration.install,
         )
 
     @staticmethod
     def git_submodules(from_path: Optional[Path] = None) -> None:
-        """ Check and/or clone git projects """
+        """Check and/or clone git projects"""
 
-        repos: Dict[str, str] = glom(
+        submodules: Dict[str, Dict[str, str]] = glom(
             Configuration.specs,
             "variables.submodules",
-            default=cast(Dict[str, str], {}),
+            default=cast(Dict[str, Dict[str, str]], {}),
         ).copy()
-        Application.gits["main"] = gitter.get_repo(".")
 
-        for name, repo in repos.items():
-            Application.gits[name] = Application.working_clone(
-                name, repo, from_path=from_path
-            )
+        main_repo = git.get_repo(".")
+        # This is to reassure mypy, but this is check is already done
+        # in preliminary checks, so it can never happen
+        if not main_repo:  # pragma: no cover
+            print_and_exit("Current folder is not a git main_repository")
 
-    def set_active_services(self) -> None:
-        self.services_dict, self.active_services = services.find_active(
-            self.compose_config
-        )
+        Application.gits["main"] = main_repo
 
-        self.enabled_services = services.get_services(
-            Configuration.services_list, default=self.active_services
-        )
+        for name, submodule in submodules.items():
+            repo = Application.working_clone(name, submodule, from_path=from_path)
+            if repo:
+                Application.gits[name] = repo
 
-        self.create_datafile()
+    def get_compose_configuration(self) -> None:
 
-    def read_composers(self) -> None:
+        compose_files: List[Path] = []
 
-        # Find configuration that tells us which files have to be read
+        MODE = f"{Configuration.stack}.yml"
+        customconf = Configuration.ABS_PROJECT_PATH.joinpath(CONTAINERS_YAML_DIRNAME)
+        angular_loaded = False
 
-        # substitute values starting with '$$'
+        def add(p: Path, f: str) -> None:
+            compose_files.append(p.joinpath(f))
 
-        myvars = {
-            "backend": Configuration.load_backend,
-            ANGULAR: Configuration.frontend == ANGULAR and Configuration.load_frontend,
-            "commons": Configuration.load_commons,
-            "extended-commons": self.extended_project is not None
-            and Configuration.load_commons,
-            "mode": f"{Configuration.stack}.yml",
-            "extended-mode": self.extended_project is not None,
-            "baseconf": CONFS_DIR,
-            "customconf": Configuration.ABS_PROJECT_PATH.joinpath(
-                CONTAINERS_YAML_DIRNAME
-            ),
-        }
+        if Configuration.load_backend:
+            add(CONFS_DIR, "backend.yml")
 
-        if self.extended_project_path is None:
-            myvars["extendedproject"] = None
+        if Configuration.load_frontend:
+            if Configuration.frontend == ANGULAR:
+                add(CONFS_DIR, "angular.yml")
+                angular_loaded = True
+
+        if SWARM_MODE:
+            add(CONFS_DIR, "swarm_options.yml")
+
+        if Application.env.get("NFS_HOST"):
+            add(CONFS_DIR, "volumes_nfs.yml")
         else:
-            myvars["extendedproject"] = self.extended_project_path.joinpath(
-                CONTAINERS_YAML_DIRNAME
-            )
+            add(CONFS_DIR, "volumes_local.yml")
 
-        compose_files = OrderedDict()
+        if Configuration.production:
+            add(CONFS_DIR, "production.yml")
+        else:
+            add(CONFS_DIR, "development.yml")
 
-        confs: Dict[str, Any] = glom(
-            Configuration.specs, "variables.composers", default={}
-        )
-        for name, conf in confs.items():
-            compose_files[name] = services.apply_variables(conf, myvars)
+            if angular_loaded:
+                add(CONFS_DIR, "angular-development.yml")
+
+        if self.extended_project and self.extended_project_path:
+            extendedconf = self.extended_project_path.joinpath(CONTAINERS_YAML_DIRNAME)
+            # Only added if exists, this is the only non mandatory conf file
+            extended_mode_conf = extendedconf.joinpath(MODE)
+            if extended_mode_conf.exists():
+                compose_files.append(extended_mode_conf)
+
+            if Configuration.load_commons:
+                add(extendedconf, "commons.yml")
+
+        if Configuration.load_commons:
+            add(customconf, "commons.yml")
+
+        add(customconf, MODE)
 
         # Read necessary files
         self.files, self.base_files = configuration.read_composer_yamls(compose_files)
-
         # to build the config with files and variables
         dc = Compose(files=self.base_files)
-        self.base_services = dc.config()
+        self.base_services = cast(ComposeConfig, dc.config().get("services", {}))
 
         dc = Compose(files=self.files)
-        self.compose_config = dc.config()
+        self.compose_config = cast(ComposeConfig, dc.config().get("services", {}))
+
+        self.set_active_services()
+
+    def set_active_services(self) -> None:
+        self.active_services = services.find_active(self.compose_config)
+
+        self.enabled_services = services.get_services(
+            Configuration.services_list,
+            Configuration.excluded_services_list,
+            default=self.active_services,
+        )
+
+        for service in self.enabled_services:
+            if service not in self.active_services:
+                print_and_exit("No such service: {}", service)
+
+        log.debug("Enabled services: {}", self.enabled_services)
+
+        self.create_datafile()
 
     def create_projectrc(self) -> None:
         templating = Templating()
@@ -664,6 +736,7 @@ class Application:
                 "production": Configuration.production,
                 "testing": Configuration.testing,
                 "services": self.active_services,
+                "envs": Configuration.environment,
             },
         )
         templating.save_template(PROJECTRC, t, force=True)
@@ -683,57 +756,84 @@ class Application:
         except FileNotFoundError:
             pass
 
-        env: Dict[str, Any] = glom(Configuration.specs, "variables.env", default={})
+        Application.env = glom(Configuration.specs, "variables.env", default={})
 
-        env["PROJECT_DOMAIN"] = Configuration.hostname
-        env["COMPOSE_PROJECT_NAME"] = Configuration.project
-        env["VANILLA_DIR"] = Path().cwd()
-        env["SUBMODULE_DIR"] = SUBMODULES_DIR.resolve()
-        env["PROJECT_DIR"] = PROJECT_DIR.joinpath(Configuration.project).resolve()
+        Application.env["PROJECT_DOMAIN"] = Configuration.hostname
+        Application.env["COMPOSE_PROJECT_NAME"] = Configuration.project
+
+        Application.env["DATA_DIR"] = str(Path("data").resolve())
+        Application.env["SUBMODULE_DIR"] = str(SUBMODULES_DIR.resolve())
+        Application.env["PROJECT_DIR"] = str(
+            PROJECT_DIR.joinpath(Configuration.project).resolve()
+        )
 
         if self.extended_project_path is None:
-            env["BASE_PROJECT_DIR"] = env["PROJECT_DIR"]
+            Application.env["BASE_PROJECT_DIR"] = Application.env["PROJECT_DIR"]
         else:
-            env["BASE_PROJECT_DIR"] = self.extended_project_path.resolve()
+            Application.env["BASE_PROJECT_DIR"] = str(
+                self.extended_project_path.resolve()
+            )
 
         if self.extended_project is None:
-            env["EXTENDED_PROJECT"] = EXTENDED_PROJECT_DISABLED
-            env["BASE_PROJECT"] = env["COMPOSE_PROJECT_NAME"]
+            Application.env["EXTENDED_PROJECT"] = EXTENDED_PROJECT_DISABLED
+            Application.env["BASE_PROJECT"] = Application.env["COMPOSE_PROJECT_NAME"]
         else:
-            env["EXTENDED_PROJECT"] = self.extended_project
-            env["BASE_PROJECT"] = env["EXTENDED_PROJECT"]
+            Application.env["EXTENDED_PROJECT"] = str(self.extended_project)
+            Application.env["BASE_PROJECT"] = Application.env["EXTENDED_PROJECT"]
 
-        env["RAPYDO_VERSION"] = __version__
-        env["PROJECT_VERSION"] = Configuration.version
-        env["CURRENT_UID"] = self.current_uid
-        env["CURRENT_GID"] = self.current_gid
-        env["PROJECT_TITLE"] = Configuration.project_title
-        env["PROJECT_DESCRIPTION"] = Configuration.project_description
-        env["PROJECT_KEYWORDS"] = Configuration.project_keywords
-        env["DOCKER_PRIVILEGED_MODE"] = "true" if Configuration.privileged else "false"
+        Application.env["RAPYDO_VERSION"] = __version__
+        Application.env["PROJECT_VERSION"] = Configuration.version
+        Application.env["CURRENT_UID"] = str(self.current_uid)
+        Application.env["CURRENT_GID"] = str(self.current_gid)
+        Application.env["PROJECT_TITLE"] = (
+            Configuration.project_title or "Unknown title"
+        )
+        Application.env["PROJECT_DESCRIPTION"] = (
+            Configuration.project_description or "Unknown description"
+        )
+        Application.env["PROJECT_KEYWORDS"] = Configuration.project_keywords or ""
 
         if Configuration.testing:
-            env["APP_MODE"] = "test"
+            Application.env["APP_MODE"] = "test"
 
-        env["CELERYBEAT_SCHEDULER"] = services.get_celerybeat_scheduler(env)
+        Application.env["CELERYBEAT_SCHEDULER"] = services.get_celerybeat_scheduler(
+            Application.env
+        )
 
-        env["DOCKER_NETWORK_MODE"] = "bridge"
+        if Configuration.load_frontend:
+            if Configuration.frontend == ANGULAR:
+                Application.env["ACTIVATE_ANGULAR"] = "1"
 
-        services.check_rabbit_password(env.get("RABBITMQ_PASSWORD"))
-        services.check_redis_password(env.get("REDIS_PASSWORD"))
-        services.check_mongodb_password(env.get("MONGO_PASSWORD"))
+        services.check_rabbit_password(Application.env.get("RABBITMQ_PASSWORD"))
+        services.check_redis_password(Application.env.get("REDIS_PASSWORD"))
+        services.check_mongodb_password(Application.env.get("MONGO_PASSWORD"))
 
-        for e in env:
+        for e in Application.env:
             env_value = os.environ.get(e)
             if env_value is None:
                 continue
-            env[e] = env_value
+            Application.env[e] = env_value
 
-        env.update(Configuration.environment)
+        Application.env.update(Configuration.environment)
+
+        if SWARM_MODE:
+
+            if not Application.env.get("SWARM_MANAGER_ADDRESS"):
+                Application.env["SWARM_MANAGER_ADDRESS"] = system.get_local_ip()
+
+            if not Application.env.get("REGISTRY_HOST"):
+                Application.env["REGISTRY_HOST"] = Application.env[
+                    "SWARM_MANAGER_ADDRESS"
+                ]
+
+            # Application.env["ACTIVATE_REGISTRY"] = "1"
+
+        if Configuration.FORCE_COMPOSE_ENGINE or not SWARM_MODE:
+            Application.env["DEPLOY_ENGINE"] = "compose"
+        else:
+            Application.env["DEPLOY_ENGINE"] = "swarm"
 
         bool_envs = [
-            # This variable is for docker-compose and is expected to be true|false
-            "DOCKER_PRIVILEGED_MODE",
             # This variable is for RabbitManagement and is expected to be true|false
             "RABBITMQ_SSL_FAIL_IF_NO_PEER_CERT",
             # These variables are for Neo4j and are expected to be true|false
@@ -742,13 +842,13 @@ class Application:
             "NEO4J_RECOVERY_MODE",
         ]
         with open(COMPOSE_ENVIRONMENT_FILE, "w+") as whandle:
-            for key, value in sorted(env.items()):
+            for key, value in sorted(Application.env.items()):
 
                 if (
                     Configuration.production
                     and key.endswith("_PASSWORD")
                     and value
-                    and len(value) < 8
+                    and len(str(value)) < 8
                 ):
                     log.warning("{} is set with a short password", key)
 
@@ -770,17 +870,6 @@ class Application:
                                 f"Deprecated value for {key}, convert {value} to 0",
                                 DeprecationWarning,
                             )
-                    elif isinstance(value, bool):
-                        if value:
-                            warnings.warn(
-                                f"Deprecated value for {key}, convert {value} to 1",
-                                DeprecationWarning,
-                            )
-                        else:
-                            warnings.warn(
-                                f"Deprecated value for {key}, convert {value} to 0",
-                                DeprecationWarning,
-                            )
 
                 if value is None:
                     value = ""
@@ -788,6 +877,8 @@ class Application:
                     value = str(value)
                 if " " in value:
                     value = f"'{value}'"
+                # if len(value) == 0:
+                #     value = f"'{value}'"
                 whandle.write(f"{key}={value}\n")
 
     def create_datafile(self) -> None:
@@ -799,8 +890,7 @@ class Application:
         data: DataFileStub = {
             "submodules": [k for k, v in Application.gits.items() if v is not None],
             "services": self.active_services,
-            "allservices": list(self.services_dict.keys()),
-            "interfaces": self.get_available_interfaces(),
+            "allservices": list(self.compose_config.keys()),
         }
 
         with open(DATAFILE, "w+") as outfile:
@@ -815,7 +905,6 @@ class Application:
                 # This is needed to let mypy understand the correct type
                 output["submodules"] = datafile.get("submodules")
                 output["services"] = datafile.get("services")
-                output["interfaces"] = datafile.get("interfaces")
                 output["allservices"] = datafile.get("allservices")
                 return output
         except FileNotFoundError:
@@ -831,25 +920,6 @@ class Application:
             return values
         return [x for x in values if x.startswith(incomplete)]
 
-    def get_available_interfaces(self) -> List[str]:
-        available_interfaces = list()
-        if self.compose_config:
-            for s in self.compose_config:
-                name = s.get("name", "")
-                if name.endswith("ui"):
-                    available_interfaces.append(name[0:-2])
-        return available_interfaces
-
-    @staticmethod
-    def autocomplete_interfaces(incomplete: str) -> List[str]:
-        d = Application.parse_datafile()
-        if not d:
-            return []
-        values = d.get("interfaces", [])
-        if not incomplete:
-            return values
-        return [x for x in values if x.startswith(incomplete)]
-
     @staticmethod
     def autocomplete_allservice(incomplete: str) -> List[str]:
         d = Application.parse_datafile()
@@ -873,7 +943,7 @@ class Application:
     def check_placeholders(self) -> Set[str]:
 
         if len(self.active_services) == 0:  # pragma: no cover
-            Application.exit(
+            print_and_exit(
                 """You have no active service
 \nSuggestion: to activate a top-level service edit your project_configuration
 and add the variable "ACTIVATE_DESIREDSERVICE: 1"
@@ -884,7 +954,7 @@ and add the variable "ACTIVATE_DESIREDSERVICE: 1"
 
         missing = set()
         for service_name in self.active_services:
-            service = self.services_dict.get(service_name)
+            service = self.compose_config.get(service_name)
 
             if service:
                 for key, value in (
@@ -915,13 +985,13 @@ and add the variable "ACTIVATE_DESIREDSERVICE: 1"
                 if key.startswith("INJECT_"):
                     key = key[len("INJECT_") :]
 
-                Application.exit(
+                print_and_exit(
                     "Missing variable: {}: cannot find a service mapping this variable",
                     key,
                 )
 
         if len(placeholders) > 0:
-            Application.exit(
+            print_and_exit(
                 "The following variables are missing in your configuration:\n\n{}"
                 "\n\nYou can fix this error by updating your .projectrc file\n",
                 "\n".join(placeholders),
@@ -937,8 +1007,8 @@ and add the variable "ACTIVATE_DESIREDSERVICE: 1"
                 log.debug("Skipping update on {}", name)
                 continue
 
-            if gitobj and not gitter.can_be_updated(name, gitobj):
-                Application.exit("Can't continue with updates")
+            if gitobj and not git.can_be_updated(name, gitobj):
+                print_and_exit("Can't continue with updates")
 
         controller_is_updated = False
         for name, gitobj in Application.gits.items():
@@ -949,11 +1019,19 @@ and add the variable "ACTIVATE_DESIREDSERVICE: 1"
                 controller_is_updated = True
 
             if gitobj:
-                gitter.update(name, gitobj)
+                git.update(name, gitobj)
 
         if controller_is_updated:
             installation_path = Packages.get_installation_path("rapydo")
-            if installation_path:
+
+            # Can't be tested on GA since rapydo is alway installed from a folder
+            if not installation_path:  # pragma: no cover
+                log.warning(
+                    "Controller is not installed in editable mode, "
+                    "rapydo is unable to update it"
+                )
+
+            elif Application.gits["do"].working_dir:
                 do_dir = Path(Application.gits["do"].working_dir)
                 if do_dir.is_symlink():
                     do_dir = do_dir.resolve()
@@ -971,12 +1049,8 @@ and add the variable "ACTIVATE_DESIREDSERVICE: 1"
                         installation_path,
                         do_dir,
                     )
-            # Can't be tested on GA since rapydo is alway installed from a folder
             else:  # pragma: no cover
-                log.warning(
-                    "Controller is not installed in editable mode, "
-                    "rapydo is unable to update it"
-                )
+                log.warning("Controller submodule folder can't be found")
 
     @staticmethod
     def git_checks(ignore_submodule: List[str]) -> None:
@@ -986,5 +1060,5 @@ and add the variable "ACTIVATE_DESIREDSERVICE: 1"
                 log.debug("Skipping checks on {}", name)
                 continue
             if gitobj:
-                gitter.check_updates(name, gitobj)
-                gitter.check_unstaged(name, gitobj)
+                git.check_updates(name, gitobj)
+                git.check_unstaged(name, gitobj)
