@@ -1,11 +1,11 @@
 from enum import Enum
 from pathlib import Path
-from typing import Any, Callable, Dict
+from typing import Callable, List
 
 import typer
 from glom import glom
 
-from controller import log
+from controller import log, print_and_exit
 from controller.app import Application, Configuration
 from controller.project import Project
 from controller.templating import Templating
@@ -17,11 +17,12 @@ class ElementTypes(str, Enum):
     component = "component"
     service = "service"
     integration_test = "integration_test"
+    workflow = "workflow"
 
 
 def get_function(
     element: ElementTypes,
-) -> Callable[[Project, str, Any, str, bool, bool], None]:
+) -> Callable[[Project, str, List[str], str, bool, bool], None]:
 
     if element == "endpoint":
         return create_endpoint
@@ -38,6 +39,9 @@ def get_function(
     if element == "integration_test":
         return create_integration_test
 
+    if element == "workflow":
+        return create_workflow
+
     # Can't be reached
     return create_service  # pragma: no cover
 
@@ -85,16 +89,18 @@ def create_template(
     template_name: str,
     target_path: Path,
     name: str,
-    services: Dict[str, str],
+    services: List[str],
     auth: str,
     force: bool,
+    project: str,
 ) -> None:
 
     if not force and target_path.exists():
-        Application.exit("{} already exists", target_path)
+        print_and_exit("{} already exists", target_path)
 
     template = templating.get_template(
-        template_name, {"name": name, "services": services, "auth": auth}
+        template_name,
+        {"name": name, "services": services, "auth": auth, "project": project},
     )
 
     templating.save_template(target_path, template, force=force)
@@ -103,7 +109,7 @@ def create_template(
 def create_endpoint(
     project_scaffold: Project,
     name: str,
-    services: Any,
+    services: List[str],
     auth: str,
     force: bool,
     add_tests: bool,
@@ -113,7 +119,14 @@ def create_endpoint(
 
     templating = Templating()
     create_template(
-        templating, "endpoint_template.py", path, name, services, auth, force
+        templating,
+        "endpoint_template.py",
+        path,
+        name,
+        services,
+        auth,
+        force,
+        project_scaffold.project,
     )
 
     log.info("Endpoint created: {}", path)
@@ -123,7 +136,14 @@ def create_endpoint(
         path = path.joinpath(f"test_endpoints_{name}.py")
 
         create_template(
-            templating, "endpoint_test_template.py", path, name, services, auth, force
+            templating,
+            "endpoint_test_template.py",
+            path,
+            name,
+            services,
+            auth,
+            force,
+            project_scaffold.project,
         )
 
         log.info("Tests scaffold created: {}", path)
@@ -132,7 +152,7 @@ def create_endpoint(
 def create_task(
     project_scaffold: Project,
     name: str,
-    services: Any,
+    services: List[str],
     auth: str,
     force: bool,
     add_tests: bool,
@@ -141,7 +161,16 @@ def create_task(
     path = path.joinpath(f"{name}.py")
 
     templating = Templating()
-    create_template(templating, "task_template.py", path, name, services, auth, force)
+    create_template(
+        templating,
+        "task_template.py",
+        path,
+        name,
+        services,
+        auth,
+        force,
+        project_scaffold.project,
+    )
 
     log.info("Task created: {}", path)
 
@@ -152,7 +181,7 @@ def create_task(
 def create_component(
     project_scaffold: Project,
     name: str,
-    services: Any,
+    services: List[str],
     auth: str,
     force: bool,
     add_tests: bool,
@@ -172,10 +201,28 @@ def create_component(
 
     cpath = path.joinpath(f"{name}.ts")
     templating = Templating()
-    create_template(templating, template_ts, cpath, name, services, auth, force)
+    create_template(
+        templating,
+        template_ts,
+        cpath,
+        name,
+        services,
+        auth,
+        force,
+        project_scaffold.project,
+    )
 
     hpath = path.joinpath(f"{name}.html")
-    create_template(templating, template_html, hpath, name, services, auth, force)
+    create_template(
+        templating,
+        template_html,
+        hpath,
+        name,
+        services,
+        auth,
+        force,
+        project_scaffold.project,
+    )
 
     log.info("Component created: {}", path)
 
@@ -185,12 +232,12 @@ def create_component(
     with open(module_path) as f:
         module = f.read().splitlines()
 
-    CNAME = "{}Component".format(name.title().replace(" ", ""))
+    normalized_name = name.title().replace(" ", "")
+    CNAME = f"{normalized_name}Component"
 
     # Add component import
-    import_line = "import {{ {} }} from '@app/components/{}/{}';".format(
-        CNAME, name, name
-    )
+    import_line = f"import {{ {CNAME} }} from '@app/components/{name}/{name}';"
+
     for idx, row in enumerate(module):
         if import_line in row:
             log.info("Import already included in module file")
@@ -242,6 +289,7 @@ def create_component(
             services,
             auth,
             force,
+            project_scaffold.project,
         )
 
         log.info("Tests scaffold created: {}", path)
@@ -250,7 +298,7 @@ def create_component(
 def create_service(
     project_scaffold: Project,
     name: str,
-    services: Any,
+    services: List[str],
     auth: str,
     force: bool,
     add_tests: bool,
@@ -262,7 +310,14 @@ def create_service(
 
     templating = Templating()
     create_template(
-        templating, "service_template.ts", path, name, services, auth, force
+        templating,
+        "service_template.ts",
+        path,
+        name,
+        services,
+        auth,
+        force,
+        project_scaffold.project,
     )
 
     log.info("Service created: {}", path)
@@ -273,7 +328,8 @@ def create_service(
     with open(module_path) as f:
         module = f.read().splitlines()
 
-    SNAME = "{}Service".format(name.title().replace(" ", ""))
+    normalized_name = name.title().replace(" ", "")
+    SNAME = f"{normalized_name}Service"
 
     # Add service import
     import_line = f"import {{ {SNAME} }} from '@app/services/{name}';"
@@ -308,14 +364,14 @@ def create_service(
 def create_integration_test(
     project_scaffold: Project,
     name: str,
-    services: Any,
+    services: List[str],
     auth: str,
     force: bool,
     add_tests: bool,
 ) -> None:
 
     if add_tests:
-        Application.exit("Add integration_test does not support --add-tests flag")
+        print_and_exit("Add integration_test does not support --add-tests flag")
 
     path = project_scaffold.p_path("frontend", "integration")
 
@@ -333,7 +389,53 @@ def create_integration_test(
 
     templating = Templating()
     create_template(
-        templating, "cypress_template.spec.ts", path, f"/{name}", services, auth, force
+        templating,
+        "cypress_template.spec.ts",
+        path,
+        f"/{name}",
+        services,
+        auth,
+        force,
+        project_scaffold.project,
     )
 
     log.info("Integration test created: {}", path)
+
+
+def create_workflow(
+    project_scaffold: Project,
+    name: str,
+    services: List[str],
+    auth: str,
+    force: bool,
+    add_tests: bool,
+) -> None:
+
+    if add_tests:
+        print_and_exit("Add workflow does not support --add-tests flag")
+
+    workflows = ["backend", "frontend", "cypress", "mypy"]
+    if name not in workflows:
+        print_and_exit("Invalid workflow name, expected: {}", ", ".join(workflows))
+
+    path = Path(".github", "workflows")
+
+    if not path.exists():
+        path.mkdir(parents=True)
+
+    filename = f"github_actions-{name}.yml"
+    path = path.joinpath(filename)
+
+    templating = Templating()
+    create_template(
+        templating,
+        filename,
+        path,
+        filename,
+        services,
+        auth,
+        force,
+        project_scaffold.project,
+    )
+
+    log.info("GitHub Actions workflow created: {}", path)
