@@ -6,13 +6,15 @@ from typing import Dict, List, Optional
 
 import typer
 
-from controller import PROJECT_DIR, __version__, gitter, log
+from controller import PROJECT_DIR, __version__, log, print_and_exit
 from controller.app import Application, Configuration
-from controller.project import NO_FRONTEND, Project
+from controller.project import NO_AUTHENTICATION, NO_FRONTEND, Project
 from controller.templating import Templating
+from controller.utilities import git
 
 
 class AuthTypes(str, Enum):
+    no = "no"
     postgres = "postgres"
     mysql = "mysql"
     neo4j = "neo4j"
@@ -90,10 +92,10 @@ def create(
 
     if extend is not None:
         if project_name == extend:
-            Application.exit("A project cannot extend itself")
+            print_and_exit("A project cannot extend itself")
 
         if not PROJECT_DIR.joinpath(extend).is_dir():
-            Application.exit("Invalid extend value: project {} not found", extend)
+            print_and_exit("Invalid extend value: project {} not found", extend)
 
     services_list: List[str] = [service.value for service in services]
     create_project(
@@ -111,12 +113,12 @@ def create(
 
     log.info("Project {} successfully created", project_name)
 
-    git_repo = gitter.get_repo(".")
+    git_repo = git.get_repo(".")
     if git_repo is None:
-        git_repo = gitter.init(".")
+        git_repo = git.init(".")
 
     print("\nYou can now init and start the project:\n")
-    current_origin = gitter.get_origin(git_repo)
+    current_origin = git.get_origin(git_repo)
 
     if current_origin is None:
         if origin_url is None:  # pragma: no cover
@@ -158,13 +160,16 @@ def create_project(
     if auth == "postgres" or auth == "mysql":
         auth = "sqlalchemy"
 
+    if auth == "no":
+        auth = NO_AUTHENTICATION
+
     if frontend == "no":
         frontend = NO_FRONTEND
 
     if not force_current:
         dirs = os.listdir(".")
         if dirs and dirs != [".git"]:
-            Application.exit(
+            print_and_exit(
                 "Current folder is not empty, cannot create a new project here.\n"
                 "Found: {}\n"
                 "Use --current to force the creation here",
@@ -201,7 +206,7 @@ def create_project(
     project_scaffold.check_invalid_characters(project_name)
 
     if project_name in project_scaffold.reserved_project_names:
-        Application.exit(
+        print_and_exit(
             "You selected a reserved name, invalid project name: {}", project_name
         )
 
@@ -217,7 +222,7 @@ def create_project(
             log.debug("Project folder already exists: {}", f)
             continue
         if not auto:
-            Application.exit("\nmkdir -p {}", f)
+            print_and_exit("\nmkdir -p {}", f)
 
         f.mkdir(parents=True, exist_ok=True)
 
@@ -230,7 +235,7 @@ def create_project(
 
     if path:
         if path not in files:
-            Application.exit("Invalid path, cannot upgrade {}", path)
+            print_and_exit("Invalid path, cannot upgrade {}", path)
         else:
             files = [path]
 
@@ -275,7 +280,7 @@ def create_project(
             log.info("Project file already exists: {}", p)
         else:
             print(f"\n{template}")
-            Application.exit(str(p))
+            print_and_exit(str(p))
 
     if not path:
         for p in project_scaffold.raw_files:
@@ -292,7 +297,7 @@ def create_project(
                 log.info("Project file already exists: {}", p)
             else:
                 # print(f"Missing file: {p}")
-                Application.exit("File is missing: {}", p)
+                print_and_exit("File is missing: {}", p)
 
 
 def parse_env_variables(envs: Optional[List[str]]) -> Dict[str, str]:
@@ -304,7 +309,7 @@ def parse_env_variables(envs: Optional[List[str]]) -> Dict[str, str]:
     for env in envs:
         e = env.split("=")
         if len(e) != 2:
-            Application.exit("Invalid env {}, expected: K1=V1", env)
+            print_and_exit("Invalid env {}, expected: K1=V1", env)
         k = e[0].upper()
         v = e[1]
         env_variables[k] = v
