@@ -1,18 +1,19 @@
 import os
 import random
 import string
-import sys
 from filecmp import cmp
 from pathlib import Path
-from typing import Any, Dict
+from typing import Dict, List, Union
 
 from jinja2 import DebugUndefined, Environment, FileSystemLoader
 from jinja2.exceptions import TemplateNotFound, UndefinedError
 
-from controller import TEMPLATE_DIR, log
+from controller import TEMPLATE_DIR, log, print_and_exit
+
+TemplateDataType = Dict[str, Union[bool, float, str, List[str], Dict[str, str], None]]
 
 
-def username(param_not_used: Any, length: int = 8) -> str:
+def username(param_not_used: str, length: int = 8) -> str:
     rand = random.SystemRandom()
     charset = string.ascii_lowercase
     random_string = rand.choice(charset)
@@ -22,9 +23,9 @@ def username(param_not_used: Any, length: int = 8) -> str:
     return random_string
 
 
-def password(param_not_used: Any, length: int = 12) -> str:
+def password(param_not_used: str, length: int = 12, symbols: str = "") -> str:
     rand = random.SystemRandom()
-    charset = string.ascii_lowercase + string.ascii_uppercase + string.digits
+    charset = string.ascii_lowercase + string.ascii_uppercase + string.digits + symbols
 
     random_string = rand.choice(charset)
     charset += string.digits
@@ -40,8 +41,7 @@ class Templating:
         self.template_dir = Path(__file__).resolve().parent.joinpath(TEMPLATE_DIR)
 
         if not self.template_dir.is_dir():
-            log.critical("Template folder not found: {}", self.template_dir)
-            sys.exit(1)
+            print_and_exit("Template folder not found: {}", self.template_dir)
 
         log.debug("Template folder: {}", self.template_dir)
         loader = FileSystemLoader([TEMPLATE_DIR, self.template_dir])
@@ -63,28 +63,17 @@ class Templating:
 
         return f"{filename}.j2"
 
-    def get_template(self, filename: str, data: Dict[str, Any]) -> str:
+    def get_template(self, filename: str, data: TemplateDataType) -> str:
         try:
             template_name = self.get_template_name(filename)
             template = self.env.get_template(template_name)
             content = str(template.render(**data))
-            # from jinja2.meta import find_undeclared_variables
-            # ast = self.env.parse(content)
-            # undefined = find_undeclared_variables(ast)
-            # if undefined:
-            #     log.critical(
-            #         'Missing variables in template: {}',
-            #         undefined
-            #     )
-            # sys.exit(1)
 
             return content
         except TemplateNotFound as e:
-            log.critical("Template {} not found in: {}", str(e), self.template_dir)
-            sys.exit(1)
+            print_and_exit("Template {} not found in: {}", str(e), self.template_dir)
         except UndefinedError as e:  # pragma: no cover
-            log.critical(e)
-            sys.exit(1)
+            print_and_exit(str(e))
 
     def save_template(self, filename: Path, content: str, force: bool = False) -> None:
 
@@ -93,8 +82,7 @@ class Templating:
                 self.make_backup(filename)
             # It is always verified before calling save_template from app, create & add
             else:  # pragma: no cover
-                log.critical("File {} already exists", filename)
-                sys.exit(1)
+                print_and_exit("File {} already exists", filename)
 
         with open(filename, "w+") as fh:
             fh.write(content)
